/********************************************************************************************************
	SOMIM, a program that can calculate the maximum mutual information and the corresponding optimal POVMs given a set of statistical operators.
	Copyright (C) 2007, 2010 K.L. Lee, J.W. Shang, W.K. Chua, S.Y. Looi and B.-G. Englert.

	This program is a free software: you can redistribute it and/or modify it under the terms of the GNU General Public License Version 3 as published by the Free Software Foundation.

	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE. 
    See the GNU General Public License for more details.

	You can refer to http://www.gnu.org/licenses/ for the GNU General Public License.
	
	Detail manual for the program could be downloaded from http://www.quantumlah.org/publications/software/SOMIM/.
		
    The code for Graphical User Interface (GUI) is following the book 'Cross-Platform GUI Programming with WxWidgets'.
********************************************************************************************************/
//******************************************************************************************************/
//	Compiled under Windows XP Service Pack 3, with wxWidgets 2.8.10, MinGW 5.1.6, MSYS 1.0.11
//
//	Compile with the following command
//		g++ MI.cpp `wx-config --libs` `wx-config --cxxflags` -o SOMIM
//	in MSYS shell.
//	
//  If you face problems running the program in a Linux environment, try
//      export LD LIBRARY PATH=/usr/local/lib
//  in command window.
//
//******************************************************************************************************/

// GUI header file for program to calculate maximum mutual information
//-----------------------------------------------
// Class definitions
//-----------------------------------------------

class myApp:public wxApp
{
	public:
		virtual bool OnInit();
};

class matrixGrid:public wxGrid
{
	public:
		matrixGrid(wxPanel *panel, int wxID);
		
		void setDimension(int x);
		void changeElement(int i,int j, double f);
		void setGrid(vector< vector<dcomp> > a, int m, int n, int ri);
};

class myPanel:public wxPanel
{
	public:    	
    	myPanel(wxFrame *frame, int x, int y);

    	wxPanel *my_panel;
    	
     wxNotebook *m_book;
    	
       wxButton *m_calcButton,
                *m_directChange;
    	
     wxRadioBox *m_rhoRadio,
	            *m_povmRadio,
    	        *m_ranRadio;
    	
     wxTextCtrl *m_displayMI,
	            *m_tol_MI,
    	        *m_output,
    	        *m_directory;
    	
   wxStaticText *m_rhoDimensionText;
	
     wxSpinCtrl *m_JTEXT,
      	  	    *m_KTEXT,
    		    *m_rhoDimension,
    		    *m_directGradChance,
		        *m_Inputrho,
		        *m_InputOrDisplayPOVM; // use this single variable to input or display POVMs
				   
	 matrixGrid *rhoGrid,
		        *povmGrid;
    	
		 MutInf *m_information;				   
		
		// event handlers to manipulate rhoGrid		   
		void ChangeRhoNumber(wxSpinEvent&);
		void ChangeRhoNumberFunction();
		void ChangeRhoDimension(wxSpinEvent&);
		void ChangeRhoDimensionFunction();
		void OnRhoCellValueChanged(wxGridEvent&);
		void UpdateRho(wxSpinEvent&);
		
		// event handlers to manipulate povmGrid
		void ChangePOVMNumber(wxSpinEvent&);
		void ChangePOVMNumberFunction();
		void ChangePOVMDimension(wxSpinEvent&);
		void ChangePOVMDimensionFunction();
		void OnPOVMCellValueChanged(wxGridEvent&);
		void UpdatePOVM(wxSpinEvent&);
		
		void OnCalc(wxCommandEvent&);
		void OnReset(wxCommandEvent&);
		void OnSave(wxCommandEvent&);
	private:
    	DECLARE_EVENT_TABLE()
};

class myFrame:public wxFrame
{
	public:
		myFrame(const wxString&, const wxPoint&, const wxSize&);
		myPanel *m_panel;
	
		int extractParameter(wxString&);
		void extractRhoElement(wxString&, int, int&);
		void extractPOVMElement(wxString&, int, int&);
		void OnImport(wxCommandEvent&);
		void OnQuit(wxCommandEvent&);
		void OnAbout(wxCommandEvent&);
	private:
		DECLARE_EVENT_TABLE()
};
//----------------------------------------------------------
//	(non-standard) event identifier defined
//----------------------------------------------------------
enum
{
    wx_Calc = 1,

    BOOKCTRL = wxID_HIGHEST + 1,
    BUTTON_Calc = wxID_HIGHEST + 2,
    BUTTON_Reset = wxID_HIGHEST + 3,
    BUTTON_Save = wxID_HIGHEST + 4,
    wxID_JTEXT = wxID_HIGHEST + 5,
    wxID_KTEXT = wxID_HIGHEST + 6,
    wxID_rhoDimension = wxID_HIGHEST + 7,
    wxID_rhoDimensionText = wxID_HIGHEST + 8,
    wxID_Inputrho = wxID_HIGHEST + 9,
    wxID_InputOrDisplayPOVM = wxID_HIGHEST + 10,
    wxID_DisplayMI = wxID_HIGHEST + 11,
    wxID_rhoGrid = wxID_HIGHEST + 12,
    wxID_povmGrid = wxID_HIGHEST + 13,
    wxID_IMPORT = wxID_HIGHEST + 14,
};
